! Simple Loop Testing Code for Depcoeff Prameterization
!  ------ JYH Penn State University, Code Consolidated, Jan 2021
!         email: jyh10@psu.edu
!         Please send any bug fixes or improvements to the author.
!
!  This code loops over crystal size for a fixed aspect ratio, temperature and supersaturation
!  The code generates the deposition coefficients for the major and minor axis of a crystal
!  The capacitance is current computed for a sphereoid. Other capacitance functions
!  should be added by the user. If a single deposition coefficient is desired, the equivalent
!  volume radius of a sphere can be used as the length scale, see below.
!
! Code use: Required input
!
! Nelson and Baker deposition coefficient equation 
!--------------------------------------------------
!
! The user must select the growth mechanism. We suggest m = 10, but other values
! between m = 1 and 15 can be selected. Suggestions based on literature:
!
! growth by dislocations (BCF theory), m = 1   (smaller crystals, low supersaturation)
! growth by ledge (2D) nucleation, m = 10      (larger crystals, all supersaturations)
! growth by stacking faults (suggested by Nelson), m = 3   (possibly low temperatures)
!
! Crystal size input
!------------------------
!
!  Lengths:
!  --------
!  alen = referenced to 1/2 the basal face diameter
!  clen = referenced to 1/2 the prism face height

!  Output:
!  -------
!  alpha_ap  = deposition coefficient for alen
!  alpha_cp  = deposition coefficient for clen
!
!  Single deposition coefficient:
!  -------------------------------
!    A single value of the deposition coefficient can be generated
!    for use in models that carry only one dimension.
!
!    Input Length:
!    -------------
!    requiv = equivalent volume radius of crystal
!    iequiv = 1 (flag to calculate single deposition coefficient)
!
!    Output:
!    -------
!    alpha_ap = alpha_cp (same value, since equiv volume sphere is used)
!
!

program depcoefftest
  
  implicit none

  integer i,iequiv
  real T0,temp,press,si,eeq,m,phi,alen,clen
  real cap,alpha_ap,alpha_cp
  real capnew,caplengtha,caplengthc,alphat
  real capacitance,ei,requiv
  real slopelg,aresid,cresid,uresid,nup1,nup2, &
       ndwn1,ndwn2

  open(20,file='depcoeffparamtest.dat')

  ! test input parameters
  !
  iequiv = 0          ! set to zero to use major and minor half-axis of a spheroid
                      ! set to one to use equivalent volume radius sphere
  m = 10.             ! ledge nucleation (birth and spread model) m = 10
                      ! dislocations m = 1
  phi = 1.0           ! crystal aspect ratio
  
  T0 = 273.15       
  temp = -15. + T0   !temp in K
  press = 10000.0    ! pressure in Pa
  si = 5./100       ! ice supersaturation in percent
  eeq = ei(temp)     ! equilibrium vapor pressure in Pa

  ! call subroutine to get parameters is fits to surface supersaturation
  call getfittingparams(m,slopelg,aresid,cresid,uresid,nup1,nup2, &
       ndwn1,ndwn2)

  do i = 10,1000
     alen = float(i)*1.e-6    ! a length
     clen = alen*phi          ! c length
     requiv = (alen**2 * clen)**(1./3.)   ! requiv for spheroid (test)
     cap = capacitance(alen,clen)    ! capacitance

     call getdepcoeffparam(si,temp,press,alen,clen,m, &
          slopelg,aresid,cresid,uresid,nup1,nup2, &
          ndwn1,ndwn2,iequiv,requiv,alpha_ap,alpha_cp,alphat)

     print*,'test ',alen*1.e6,clen*1.e6,alpha_ap,alpha_cp
     write(20,'(16e12.4)')alen*1.e6,clen*1.e6,cap*1.e6,alpha_ap,alpha_cp
  enddo
  
  stop
end program depcoefftest

!----------------------------------------------------------------------
! SUBROUTINE GETFITTINGPARAMS
!
! This subroutine gets the various parameters used in the fits for the
! surface supersaturation based on the chosen value of M
!     JYH, Penn State University, May 15 2019
!
!  Input: m (parameter in Nelson and Baker (1996) approximation for
!            the deposition coefficient)
!
!  Output: slopelg,aresid,cresid,uresid,nup1,nup2,ndwn1,ndwn2
!          (Coefficients in surface supersaturation polynomial fit)
!
!----------------------------------------------------------------------
subroutine getfittingparams(m,slopelg,aresid,cresid,uresid,nup1,nup2, &
     ndwn1,ndwn2)

  implicit none
  real m   ! input, m in equation for deposition coefficient
  real slopelg,aresid,cresid,uresid,nup1,nup2, &    ! output, fitting parameters
       ndwn1,ndwn2
  
  slopelg = 0.832
  slopelg = 0.1532 + 0.49078 * m - 0.18002 * m**2 + 0.04165 * m**3 - &
       0.0061541 * m**4 + 0.00058009 * m**5 - 3.3791e-05 * m**6 + &
       1.1092e-06 * m**7 - 1.5691e-08 * m**8
  aresid = 0.46192 - 0.093575 * m + 0.012798 * m**2 - &
       0.0008756 * m**3 + 2.2466e-05 * m**4
  cresid = aresid
  uresid = 0.25872 - 0.057957 * m + 0.0079602 * m**2 - &
       0.00052905 * m**3 + 1.3145e-05 * m**4
  
  nup1 = -0.69061 + 10.765 * m - 9.1666 * m**2 + 3.8888 * m**3 - &
       0.91779 * m**4 + 0.12544 * m**5 - 0.0098291 * m**6 +  &
       0.00040868 * m**7 - 6.9744e-06 * m**8
  
  nup2 = 5.1239 - 19.204 * m + 15.236 * m**2 - 6.4802 * m**3 +  &
       1.5151 * m**4 - 0.20317 * m**5 + 0.015601 * m**6 - &
       0.00063752 * m**7 + 1.0736e-05 * m**8
  
  ndwn1 = -0.86527 - 0.03202 * m + 0.027342 * m**2 - 0.04378 * m**3 &
       + 0.014734 * m**4 - 0.0021036 * m**5 + 0.0001431 * m**6 &
       - 4.317e-06 * m**7 + 3.9242e-08 * m**8
  
  ndwn2 = -4.692 + 12.47 * m - 9.8936 * m**2 + 4.1881 * m**3 &
       - 1.0003 * m**4 + 0.14011 * m**5 - 0.011359 * m**6 &
       + 0.00049122 * m**7 - 8.7303e-06 * m**8      
  
  return
end subroutine getfittingparams

!-------------------------------------------------------------------------
! SUBROUTINE DEPCOEFFPARAM
!
! This subroutine finds the deposition coefficients for spheroids. 
! The theory follows that of Zhang and Harrington (2014)
! however the iteration to find the alpha values is replaced
! by a parameterization. This version allows for a choice of m.
!
!  Article: Harrington et al. (2021, J. Atmos. Sci.)
!
!   JYH, Penn State University, May 15, 2019
!
!  Input:
!  ------
!    ssi:  ambient ice supersaturation (units: fraction)
!    tkelv: Temperature  (units: Kelvin)  
!    ppa:   Atmospheric pressure (units: Pa)
!    alen:  spheroid a-semi-dimension (half diameter basal plane) (units: m)
!    clen:  spheroid c-semi-dimension (half prism plane height) (units: m)
!    iequiv:  = 1  (calculation done for equivalent volume sphere)
!             = any other value for spheroidal calculation
!    requiv:  equivalent volume sphere radius (units: m)
!    m:       parameter in Nelson and Baker (1996) approximation for
!            the deposition coefficient
!    Coefficients: slopelg,aresid,cresid,uresid,nup1,nup2,ndwn1,ndwn2
!           (Coefficients in surface supersaturation polynomial fit. 
!            Caculated in subroutine getfittingparams)
!
!  Output:
!  -------
!      alpha_ap:  Deposition coefficient for a-axis
!      alpha_cp:  Deposition coefficient for c-axis
!                 For equivalent volume sphere, alpha_ap = alpha_cp
!      alpha_t:   Thermal accomodation coefficient, simply set to 0.96
!
!-------------------------------------------------------------------------

      subroutine getdepcoeffparam(ssi,tkelv,ppa, &
           alen,clen,m,slopelg,aresid,cresid,uresid,nup1,nup2, &
           ndwn1,ndwn2,iequiv,requiv,alpha_a,alpha_c,alphat)

      implicit none
      integer iequiv
      real ssi,tkelv,pmb,C3a,Dv
      real mfp,vel,Pi,Mw,Runiv,m,ppa,tcels
      real cp,xk,rv,ei0,tkelvin,Rd,rhoa
      real cap,capnew,caplengtha,caplengthc,phi,C3c,alen,clen
      real alpha_a,alpha_c,scrit_a,scrit_c,S_a,S_c,ltin,lt
      real slocal_a,slocal_c,alphat,capa,capc,suamb,pratio,rlambda,delv
      real amn,cmn,capacitance,sisloc_c,sisloc_a,gtp_stand
      real aresid,cresid,slocal_diff_a,slrata_interp,slocal_ap
      real slocal_diff_c,slratc_interp,slocal_cp
      real scrat_low,scrat_hi,dsc,ei
      real scratio_a,scratio_c,slrata_interp_lim,sldiffa_upratio
      real slratc_interp_lim,sldiffc_upratio,To,tempsc
      real sa_temp,sc_temp,mvalin,Lsub,gtp1,gtp2
      real nup1,nup2,ndwn1,ndwn2,uresid,slopelg
      real requiv,Lv,Lf
 
      parameter(Pi = 3.14159, Mw = 0.01802, Runiv = 8.314, Rv=461.5)
      parameter(Rd = 287.,To=273.15, cp = 1004.)

      if(iequiv.eq.1)then   ! assumed equiv vol sphere
         alen = requiv
         clen = requiv
      endif

      tcels = tkelv - 273.15
      ei0=ei(tkelv)
      tempsc = min(max(tkelv,To-60.),To-10.)    ! limit T used for scrit to -10 to -60C fit.

      ! get critical supersaturations from polynomial fits
      call getscrit(tkelv,sa_temp,sc_temp)
      scrit_a = sa_temp
      scrit_c = sc_temp
      if(iequiv.eq.1) scrit_a = 2./3.*scrit_a + 1./3.*scrit_c 

      pmb = ppa/100.      ! pressure is hPa units

      !  Calculation of enthalpies
      Lv = 2.5008e6*(273.15/tkelv)**(0.167+3.67e-4*tkelv)
      Lf = 0.3337e6+(2031.0-10.467*(tkelv-273.15))*(tkelv-273.15)
      Lsub = Lv+Lf

      ! Diffusivities
      Dv = 2.11e-5*(tkelv/273.15)**1.94 * 1013.25/pmb  ! vapor diff. (m^2/s)
      xk = 2.3823e-2 +7.1177e-5*(tkelv-273.15) ! thermal conductivity in W/m/k
      gtp1 = Rv*tkelv/(Dv*ei0)
      gtp2 = (Lsub**2.0/(xk*Rv*tkelv**2.0)) - &
           (Lsub/(xk*tkelv))
      gtp_stand = 1.0/(gtp1+gtp2)                      ! effective diffusivity
      mfp = 6.6e-8*(1013.25/pmb)*(tkelv/273.15)        ! mean free path in m
      vel = sqrt(8.0*Runiv*tkelv/(Pi*Mw))              ! molecular speed(m/s)
      S_a = abs(ssi/scrit_a)                           ! critical sat ratio for a axis
      S_c = abs(ssi/scrit_c)                           ! critical sat ratio for c axis

      pratio = 1013.15/pmb
      rlambda = 6.6e-6 * pratio * (tkelv / 293.15)/100. ! mean free path of ai
      delv = rlambda

      rhoa = ppa/(Rd*tkelv)
      alphat = 0.96                ! thermal accomodation coefficient
      lt = xk/(alphat*rhoa*cp)*sqrt(2.*Pi/(Rd*tkelv))
      ltin = lt

      amn = alen + delv
      cmn = clen + delv
      cap = capacitance(alen,clen)
      capnew = capacitance(amn,cmn)
      caplengtha = alen*clen/cap
      caplengthc = alen**2/cap
      capa = caplengtha
      capc = caplengthc

      C3a = (vel*caplengtha*cap/capnew)/(4.*dv)       
      C3c = (vel*caplengthc*cap/capnew)/(4.*dv)

      ! Get alpha for a-axis using parameteriztion
      !_____________________________________________________

      sisloc_a = dv/(gtp_stand*rv*tkelv/ei0) /(1./(1.+C3a))

      slocal_diff_a = ssi/sisloc_a
      scratio_a = min(max(slocal_diff_a/scrit_a,scrat_low),scrat_hi)
      slrata_interp = 10.**(slopelg*alog10(slocal_diff_a/scrit_a))
      slrata_interp_lim = max(min(slrata_interp,1.0) , &
           1.0/(sisloc_a))
      sldiffa_upratio = max(0.0,slrata_interp-1.) + 1.
      slrata_interp = max(slrata_interp , &
           1.0/(sisloc_a)) &
           + aresid*slrata_interp * &
           (slrata_interp/(1./sisloc_a))**(ndwn1) * &
           min((slrata_interp/(1./sisloc_a))**(ndwn2),1.0) &
           - uresid*slrata_interp * slrata_interp_lim**nup1 * &
           sldiffa_upratio**(nup2)
      slocal_ap = slocal_diff_a/slrata_interp
      alpha_a=(slocal_ap/scrit_a)**m * tanh((scrit_a/slocal_ap)**m)
      slocal_a = slocal_ap
      alpha_a = max(min(alpha_a,1.0),1.e-6)

      ! Get alpha for c-axis next from parameterization
      !_____________________________________________________

      if(iequiv.ne.1)then

         sisloc_c = dv/(gtp_stand*rv*tkelv/ei0) /(1./(1.+C3c))

         slocal_diff_c = ssi/sisloc_c
         scratio_c = min(max(slocal_diff_c/scrit_c,scrat_low),scrat_hi)
         slratc_interp = 10.**(slopelg*alog10(slocal_diff_c/scrit_c))
         slratc_interp_lim = max(min(slratc_interp,1.0) , &
              1.0/(sisloc_c))
         sldiffc_upratio = max(0.0,slratc_interp-1.) + 1.
         slratc_interp = max(slratc_interp, &
              1./sisloc_c)     &
              + cresid*slratc_interp * &
              (slratc_interp/(1./sisloc_c))**(ndwn1) * &
              min((slratc_interp/(1./sisloc_c))**(ndwn2),1.0) &
!           - 0.11*slratc_interp * slratc_interp_lim**nup1 * &
              - uresid*slratc_interp * slratc_interp_lim**nup1 * &
              sldiffc_upratio**(nup2)
         slocal_cp = slocal_diff_c/slratc_interp
         alpha_c=(slocal_cp/scrit_c)**m * tanh((scrit_c/slocal_cp)**m)
         alpha_c = max(min(alpha_c,1.0),1.e-6)
         slocal_c = slocal_cp
      else
         alpha_c = alpha_a
         slocal_c = slocal_a
      endif

      return
    end subroutine getdepcoeffparam

!-----------------------------------------------------------------------
! SUBROUTINE GETSCRIT
!
! This subroutine finds the critical (characteristic) supersaturations
! for the basal (c-axis) and prism (a-axis) facets based on polynomial
! fits to available data
!
!  JYH Penn State University, May 15, 2019
!
!  Article: Harrington et al. (2019, J. Atmos. Sci.)
!
!------------------------------------------------------------------------

subroutine getscrit(temp,sa,sc)
  
  implicit none
  
  real temp,sa,sc
  real dt,x
  
  dt = temp-273.15
  x = max(-70.,min(dt,-1.0))
  if(dt.lt.-30.0)then
     ! original magee (high scrit) 
     sc = 1.8115 + 0.15585*x + 0.011569*x**2
     sa = 1.8115 + 0.15585*x + 0.011569*x**2
     
     ! with bailey and hallett
     sc = 3.7955 + 0.10614*x + 0.0075309*x**2
     sa = sc
     ! modified magee
     !sc = 3.3649 + 0.13266*x + 0.008959*x**2
     !sa = sc
  else if(dt.ge.-30.0.and.dt.le.-22.0)then
     sc = 753.63 + 105.97 * x + 5.5532 * x**2 + 0.12809 * x**3  &
          + 0.001103 * x**4
  else if (dt.gt.-22.0.and.dt.le.-1.0) then
     sc = 1.1217 + 0.038098 * x - 0.083749 * x**2  &
          - 0.015734 * x**3 - 0.0010108 * x**4  &
          - 2.9148e-05 * x**5 - 3.1823e-07 * x**6
  endif
  if(dt.ge.-30.0.and.dt.le.-22.0)then
     sa =  -0.71057 - 0.14775*x + 0.0042304*x**2
  elseif (dt.gt.-22.0.and.dt.le.-15.0)then
     sa = -5.2367 - 1.3184*x - 0.11066*x**2 - 0.0032303*x**3
  elseif (dt.gt.-15.0.and.dt.le.-10.0)then
     !sa = 0.755 + 0.03325*x + 0.00125*x**2   ! fit to Woods
     sa = 0.34572 - 0.0093029*x + 0.00030832*x**2 ! fit to Nelson & Night
  elseif (dt.gt.-10.0.and.dt.le.-1.0)then
     !          sa = 0.37445 + 0.029636*x + 0.011575*x**2 &      ! Wood
     !               + 0.00069444*x**3
     sa = 0.34572 - 0.0093029*x + 0.00030832*x**2 ! Nelson & Knight
  endif
     
  sa = sa/100.
  sc = sc/100.
  
  return
end subroutine getscrit


! FOLLOWING SUBROUTINES PROVIDED FOR TEST CODE.
!-------------------------------------------------

function capacitance(a,c)
!     The changing capacitance accounts for the changing 
!     shape of the crystal.

  implicit none
  real phi,capacitance,a,c
  phi = c/a
  
!     Oblate Spheroid
  if (phi.lt.1) then
     capacitance = (a**2 - c**2)**0.5 / asin((1-phi**2)**0.5)

!     Prolate Spheroid
  elseif (phi.gt.1) then
     capacitance = (c**2 - a**2)**0.5 / log(phi+phi*  &
          (1-phi**-2)**0.5)
     
!     Sphere
  elseif (phi.eq.1) then
     capacitance = a
  endif
  
  return
end function capacitance

!**********************************************************************
!     This function computes the equilibrium vapor pressure
!     over ice. Arguement is in K and the units returned
!     are mks
!**********************************************************************
    real function ei(tmp)

      real ci(0:8)
      
      data ci/611.4327,50.27041,1.875982,.4158303e-1,.5992408e-3, &
           .5743775e-5,.3566847e-7,.1306802e-9,.2152144e-12/
      
      t          = tmp-273.15
      x          = max(-80.,t)
      eisair     = ci(0)+x*(ci(1)+x*(ci(2)+x*(ci(3)+x*(ci(4)+x*(ci(5)+ &
           x*(ci(6)+x*(ci(7)+x*ci(8))))))))
      
      ei = eisair
      
      return
    end function ei
